
yarip.Overlay = function()
{
	this.enabled = false;
	this.active = false;
	this.noFlicker = false;

	this.control = null;
	this.loader = null;
	this.handler = null;
	this.highlighter = null;
	this.pages = null;
	this.mode = null;
	this.deep = null;

	this.start = function(event)
	{
		if (event.button != 0) return;

		var doc = content.document;
		if (!this.enabled)
		{
			this.toggleEnabled(true);
			if (doc)
			{
				if (this.loader.load(doc, this.pages)) this.setStatus(doc, "found");
				if (doc.body) doc.body.setAttribute("status", "show");
			}
			return;
		}
		if (this.active)
		{
			this.stop(false);
			return;
		}
		if (!doc) return;

		this.handler = null;
		if (!doc.defaultView.handler) return;
		this.handler = doc.defaultView.handler;

		if (!this.handler.start(doc, this.pages, this.mode))
		{
			this.stop();
			return;
		}

		this.active = true;
		this.setStatus(doc, "started");
	}

	this.stop = function(noStateChange)
	{
		if (!this.active || !this.handler) return;

		this.handler.stop();
		this.active = false;

		if (noStateChange) return;
		this.setStatus(content.document, "stopped");
	}

	this.save = function()
	{
		this.stop();
		this.control.savePages(this.pages);
	}

	this.reset = function(mode, noStateChange)
	{
		this.stop();
		this.toggleEnabled(true);

		var node = document.popupNode;
		if (!node) return;
		var doc = node.ownerDocument;
		if (!doc) return;
		if (typeof mode != "number")
		{
			var aAddress = yarip.getAddressArray(doc, this.pages);
			for (var i = aAddress.length - 1; i >= 0; i--) this.pages[aAddress[i]] = null;
			if (noStateChange) return;
			return;
		}

		var address = yarip.getAddress(doc, mode);
		if (!address || !this.pages || !this.pages[address]) return;
		this.pages[address] = null;
	}

	this.reload = function()
	{
		this.stop();
		this.toggleEnabled(true);
		this.pages = this.control.loadPages();
	}

	this.whitelist = function(doc, node)
	{
		this.stop();
		this.toggleEnabled(true);

		if (!node) return;

		var address = yarip.getAddress(doc, this.mode);
		if (!doc || !address) return;

		var xValue = yarip.calculateXPath(node);
		if (!xValue) return;
		xValue = prompt("Edit XPath for whitelisting", xValue);
		if (!xValue) return;

		if (!this.pages[address]) this.pages[address] = new yarip.Page(address);
		this.pages[address].whitelist.addXPath(new yarip.XPath(xValue, this.deep));
	}

	this.blacklist = function(doc, node)
	{
		this.stop();
		this.toggleEnabled(true);

		if (!node) return;

		var address = yarip.getAddress(doc, this.mode);
		if (!address) return;

		var xValue = yarip.calculateXPath(node);
		if (!xValue) return;
		xValue = prompt("Edit XPath for blacklisting", xValue);
		if (!xValue) return;

		yarip.removeByXPath(doc, this.pages, address, new yarip.XPath(xValue, this.deep), true);
	}

	this.startHighlighting = function(node)
	{
		if (!node) return;

		this.highlighter.element = node;
		this.highlighter.select();
	}

	this.stopHighlighting = function()
	{
		this.highlighter.reset();
	}

	this.setMode = function(mode, clear)
	{
		if (typeof mode != "number") return;
		this.stop();

		if (clear)
		{
			document.getElementById("yarip-page-menuitem").setAttribute("checked", "false");
			document.getElementById("yarip-fqdn-menuitem").setAttribute("checked", "false");
			document.getElementById("yarip-sld-menuitem").setAttribute("checked", "false");
			document.getElementById("yarip-user-menuitem").setAttribute("checked", "false");
		}

		switch (mode)
		{
			case yarip.MODES.page:
				document.getElementById("yarip-page-menuitem").setAttribute("checked", "true");
				break;

			case yarip.MODES.fqdn:
				document.getElementById("yarip-fqdn-menuitem").setAttribute("checked", "true");
				break;

			case yarip.MODES.sld:
				document.getElementById("yarip-sld-menuitem").setAttribute("checked", "true");
				break;

			case yarip.MODES.user:
				var doc = content.document;
				var address = yarip.getAddress(doc, this.mode);
				if (!address)
				{
					yarip.address = null;
					if (this.mode != yarip.MODES.user) this.setMode(this.mode, true);
					return;
				}
				address = prompt("Edit address", address);
				if (!address)
				{
					yarip.address = null;
					if (this.mode != yarip.MODES.user) this.setMode(this.mode, true);
					return;
				}
				yarip.address = address;
				document.getElementById("yarip-user-menuitem").setAttribute("checked", "true");
		}

		this.mode = mode;
	}

	this.toggleEnabled = function(enabled)
	{
		if (enabled != true && enabled != false) enabled = !this.control.getValue(yarip.ROOT + "enabled", true, yarip.BOOLEAN);

		if (!enabled) this.stop(true);

		this.observer.disable();

		this.control.setValue(yarip.ROOT + "enabled", enabled, yarip.BOOLEAN);
		this.enabled = enabled;

		this.observer.enable();

		if (!enabled) this.toggleNoFlicker(false);

		document.getElementById("yarip-enabled-menuitem").setAttribute("checked", "" + enabled);
		this.setStatus(content.document, enabled ? "enabled" : "disabled");
	}

	this.updateEnabled = function()
	{
		this.observer.disable();

		var enabled = this.control.getValue(yarip.ROOT + "enabled", true, yarip.BOOLEAN);
		this.toggleEnabled(enabled);

		this.observer.enable();
	}

	this.toggleNoFlicker = function(noFlicker)
	{
		if (noFlicker == this.noFlicker) return;

		if (noFlicker != true && noFlicker != false) noFlicker = !this.control.getValue(yarip.ROOT + "noFlicker", true, yarip.BOOLEAN);
		this.noFlicker = noFlicker && this.enabled;
		this.control.setValue(yarip.ROOT + "noFlicker", this.noFlicker, yarip.BOOLEAN);

		var ios = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
		var uri = ios.newURI("chrome://yarip/content/inject.css", yarip.CHARSET, null); // URI, CHARSET, BASEURI
		var sss = Components.classes["@mozilla.org/content/style-sheet-service;1"].getService(Components.interfaces.nsIStyleSheetService);

		if (this.noFlicker) { if (!sss.sheetRegistered(uri, sss.USER_SHEET)) sss.loadAndRegisterSheet(uri, sss.USER_SHEET); }
		else { if (sss.sheetRegistered(uri, sss.USER_SHEET)) sss.unregisterSheet(uri, sss.USER_SHEET); }
	}

	this.toggleDeep = function(deep)
	{
		if (deep == this.deep) return;

		if (deep != true && deep != false) deep =  !this.deep;
		this.deep = deep;

		document.getElementById("yarip-deep-menuitem").setAttribute("checked", "" + this.deep);
	}

	this.observer = null;

	this.handleEvent = function(event)
	{
		switch(event.type)
		{
			case "load":
				this.stop();

				this.control = new yarip.Control(this);
				this.pages = this.control.loadPages();
				this.loader = new yarip.Loader(this);
				this.highlighter = new yarip.Handler(this);

				this.observer = new yarip.Observer(this.control, yarip.UPDATE_ENABLED, yarip.ROOT, "enabled");
				this.observer.register();

				this.toggleEnabled(this.control.getValue(yarip.ROOT + "enabled", true, yarip.BOOLEAN));
				this.toggleNoFlicker(this.control.getValue(yarip.ROOT + "noFlicker", false, yarip.BOOLEAN));
				this.toggleDeep(true);

				this.setMode(yarip.MODES.fqdn, true);

				document.getElementById("appcontent").addEventListener("DOMContentLoaded", this, false);
				document.getElementById("contentAreaContextMenu").addEventListener("popupshowing", this, false);
				gBrowser.mPanelContainer.addEventListener("select", this, false);

				break;

			case "unload":
				this.stop();
				this.save();
				break;

			case "DOMContentLoaded":
				this.stop();

				this.toggleDeep(true);

				var doc = event.originalTarget;
				if (!doc) return;

				var inFrame = doc.defaultView.frameElement != null;

				// HANDLER
				if (!inFrame)
				{
					doc.defaultView.handler = new yarip.Handler(this);
					if (this.enabled) this.setStatus(doc, "stopped");
				}
				else
				{
					var rootDoc = doc;
					while (rootDoc.defaultView.frameElement) rootDoc = rootDoc.defaultView.frameElement.ownerDocument; // get root doc
					if (!rootDoc.defaultView.handler) break;
					var handler = new yarip.Handler(this, doc);
					rootDoc.defaultView.handler.addHandler(handler);
				}

				// LOADER
				if (doc.nodeName == "#document" && this.enabled && this.pages)
				{
					if (this.loader.load(doc, this.pages))
					{
						this.toggleNoFlicker(true);
						this.setStatus(doc, "found");
					}
					else if (!inFrame) this.toggleNoFlicker(false);

					if (this.mode == yarip.MODES.user) this.setMode(yarip.MODES.fqdn, true);
				}
				else this.toggleNoFlicker(false);

				if (doc.body) doc.body.setAttribute("status", "show");
				break;

			case "DOMMenuItemActive":
				this.toggleNoFlicker(false);
				break;

			case "keypress":
				if (event.ctrlKey) this.toggleNoFlicker(false);
				break;

			case "popupshowing":
				var node = document.popupNode;
				var doc = node.ownerDocument;

				var hidden = yarip.getAddress(doc, yarip.overlay.mode) == null;
				if (!hidden) document.getElementById("yarip-element-menu").setAttribute("label", node.localName.toUpperCase());
				document.getElementById("yarip-menu").hidden = hidden;
				document.getElementById("yarip-menu-sep").hidden = hidden;

				var yaripFrameMenu = document.getElementById("yarip-frame-menu");
				var inFrame = gContextMenu.inFrame;
				var disabled = false;
				if (inFrame)
				{
					var frame = node.ownerDocument.defaultView.frameElement;
					yaripFrameMenu.setAttribute("label", frame.localName.toUpperCase());
					disabled = yarip.getAddress(frame.ownerDocument, yarip.overlay.mode) == null;
				}
				yaripFrameMenu.hidden = !inFrame;
				if (disabled) yaripFrameMenu.setAttribute("disabled", true);
				else yaripFrameMenu.removeAttribute("disabled");
				break;

			case "select":
				this.setStatus(gBrowser.getBrowserAtIndex(gBrowser.mTabContainer.selectedIndex).contentDocument);
		}
	}

	this.setStatus = function(doc, status)
	{
		if (this.enabled)
		{
			if (!status)
			{
				status = doc.defaultView.myStatus;
				switch (status)
				{
					case "enabled":
					case "stopped":
					case "found":
						break;

					case "disabled":
					case "started": if (this.active) break;
					default:
						status = "enabled";
				}
			}
		}
		else status = "disabled";

		doc.defaultView.myStatus = status;
		document.getElementById("yarip-statusbarpanel").setAttribute("status", status);
	}
}
